@@ -117,30 +117,31 @@ class Data1D:
         """
         np.ndarray : x data (possibly masked)
         """
-        if self._x.size > 0:
+        if self._mask is not None and self._x.size:
             return self._x[self.mask]
-        else:
-            return self._x
+        return self._x
 
     @property
     def y(self):
         """
         np.ndarray : y data (possibly masked)
         """
-        if self._y.size > 0:
+        if self._mask is not None and self._y.size:
             return self._y[self.mask]
-        else:
-            return self._y
+        return self._y
 
     @property
     def x_err(self):
         """
         np.ndarray : x uncertainty (possibly masked)
         """
-        if self._x_err is not None:
+        if (
+            self._x_err is not None
+            and self._mask is not None
+            and self._x_err.size
+        ):
             return self._x_err[self.mask]
-        else:
-            return self._x_err
+        return self._x_err
 
     @x_err.setter
     def x_err(self, x_err):
@@ -154,10 +155,13 @@ class Data1D:
         """
         uncertainties on the y data (possibly masked)
         """
-        if self._y_err is not None:
+        if (
+            self._y_err is not None
+            and self._mask is not None
+            and self._y_err.size
+        ):
             return self._y_err[self.mask]
-        else:
-            return self._y_err
+        return self._y_err
 
     @property
     def mask(self):
@@ -375,7 +379,7 @@ class Data1D:
         if self.x_err is not None:
             self._x_err = self.x_err[sorted]
 
-    def save(self, f):
+    def save(self, f, header=None):
         """
         Saves the data to file. Saves the data as 4 column ASCII.
 
@@ -385,13 +389,23 @@ class Data1D:
             File to save the dataset to.
 
         """
+        if header is not None:
+            _header = header
+        elif len(self.metadata):
+            _header = str(self.metadata)
+        else:
+            _header = ""
+
         np.savetxt(
-            f, np.column_stack((self._x, self._y, self._y_err, self._x_err))
+            f,
+            np.column_stack((self._x, self._y, self._y_err, self._x_err)),
+            header=_header,
         )
 
     def load(self, f):
         """
-        Loads a dataset from file. Must be 2 to 4 column ASCII.
+        Loads a dataset from file, and overwrites existing data.
+        Must be 2 to 4 column ASCII.
 
         Parameters
         ----------
