@@ -36,7 +36,7 @@ def component_class(component):
         return StackNode
 
 
-class Node(object):
+class Node:
     def __init__(self, data, model, parent=QtCore.QModelIndex()):
         self._data = data
         self._children = []
@@ -149,7 +149,7 @@ class Node(object):
 
 class ParNode(Node):
     def __init__(self, data, model, parent=QtCore.QModelIndex()):
-        super(ParNode, self).__init__(data, model, parent)
+        super().__init__(data, model, parent)
 
     @property
     def parameter(self):
@@ -175,12 +175,20 @@ class ParNode(Node):
                     return QtCore.Qt.Unchecked
             else:
                 return None
+        elif role == QtCore.Qt.ToolTipRole:
+            if column in [1, 2]:
+                p = self.parameter
+                return getattr(p, "units", None)
+            elif column == 3:
+                return "Lower limit for the parameter"
+            elif column == 4:
+                return "Upper limit for the parameter"
 
         if role == QtCore.Qt.DisplayRole:
             p = self.parameter
             d = [
                 p.name,
-                p.value,
+                float(p.value),
                 p.stderr,
                 p.bounds.lb,
                 p.bounds.ub,
@@ -224,7 +232,7 @@ class ParNode(Node):
 
 class ParametersNode(Node):
     def __init__(self, data, model, parent=QtCore.QModelIndex()):
-        super(ParametersNode, self).__init__(data, model, parent)
+        super().__init__(data, model, parent)
         for p in data:
             if isinstance(p, Parameters):
                 n = ParametersNode(p, model, self)
@@ -258,7 +266,7 @@ class PropertyNode(Node):
     def __init__(
         self, data, model, parent=QtCore.QModelIndex(), validators=()
     ):
-        super(PropertyNode, self).__init__(data, model, parent)
+        super().__init__(data, model, parent)
         # here self._data is the attribute name
         self.attribute_type = type(getattr(parent._data, data))
         self.validators = validators
@@ -331,7 +339,7 @@ class ComponentNode(Node):
             If `flat is True`, then this superclass will flatten out all
             the parameters in a model and append them as child items.
         """
-        super(ComponentNode, self).__init__(data, model, parent)
+        super().__init__(data, model, parent)
 
         if flat:
             for par in flatten(data.parameters):
@@ -382,7 +390,7 @@ class ComponentNode(Node):
 
 class StructureNode(Node):
     def __init__(self, data, model, parent=QtCore.QModelIndex()):
-        super(StructureNode, self).__init__(data, model, parent)
+        super().__init__(data, model, parent)
         for component in data:
             self.appendChild(
                 component_class(component)(component, model, self)
@@ -447,11 +455,19 @@ class StructureNode(Node):
         self._model.endInsertRows()
 
 
+# the child note where structures start appearing in a ReflectModelNode
+STRUCT_OFFSET = 4
+
+
 class ReflectModelNode(Node):
     def __init__(self, data, model, parent=QtCore.QModelIndex()):
-        super(ReflectModelNode, self).__init__(data, model, parent=parent)
+        super().__init__(data, model, parent=parent)
 
-        self.constantdq_q = True
+        # pointwise/constant dq/q choice is kept in the ReflectModel
+        if data.dq_type == "constant":
+            self.constantdq_q = True
+        elif data.dq_type == "pointwise":
+            self.constantdq_q = False
 
         # deal with scale factors
         if isinstance(data, ReflectModel):
@@ -464,6 +480,8 @@ class ReflectModelNode(Node):
         self.appendChild(n)
         n = ParNode(data.dq, model, self)
         self.appendChild(n)
+        n = ParNode(data.q_offset, model, self)
+        self.appendChild(n)
 
         if isinstance(data, ReflectModel):
             n = StructureNode(data.structure, model, self)
@@ -475,7 +493,10 @@ class ReflectModelNode(Node):
 
     @property
     def structures(self):
-        return [self.child(i)._data for i in range(3, self.childCount())]
+        return [
+            self.child(i)._data
+            for i in range(STRUCT_OFFSET, self.childCount())
+        ]
 
     def data(self, column, role=QtCore.Qt.EditRole):
         if role == QtCore.Qt.CheckStateRole and column == 1:
@@ -495,7 +516,7 @@ class ReflectModelNode(Node):
 
     def remove_structure(self, row):
         # don't remove the last structure
-        if row < 3 or len(self.structures) < 2:
+        if row < STRUCT_OFFSET or len(self.structures) < 2:
             return
 
         # you have to be a mixedreflectmodel because there's more than one
@@ -514,8 +535,8 @@ class ReflectModelNode(Node):
             # do this before the Structure is popped.
             self._model.unlink_dependent_parameters(self.child(row))
 
-            structures.pop(row - 3)
-            scales.pop(row - 3)
+            structures.pop(row - STRUCT_OFFSET)
+            scales.pop(row - STRUCT_OFFSET)
             sf = possibly_create_parameter(scales[0], name="scale")
             new_model = ReflectModel(
                 structures[0], scale=sf, bkg=orig_model.bkg, dq=orig_model.dq
@@ -543,12 +564,14 @@ class ReflectModelNode(Node):
 
         # pop the structure and scale factor nodes
         self.popChild(row)
-        structures.pop(row - 3)
-        scales.pop(row - 3)
+        structures.pop(row - STRUCT_OFFSET)
+        scales.pop(row - STRUCT_OFFSET)
         self._model.endRemoveRows()
 
-        self._model.beginRemoveRows(self.child(0).index, row - 3, row - 3)
-        self.child(0).popChild(row - 3)
+        self._model.beginRemoveRows(
+            self.child(0).index, row - STRUCT_OFFSET, row - STRUCT_OFFSET
+        )
+        self.child(0).popChild(row - STRUCT_OFFSET)
         self._model.endRemoveRows()
 
     def insert_structure(self, row, structure):
@@ -571,24 +594,26 @@ class ReflectModelNode(Node):
             return
 
         # already a mixed model
-        # we can't insert at a lower place than the 3rd row
-        row = max(row, 3)
+        # we can't insert at a lower place than the 4rd row
+        row = max(row, 4)
         self._model.beginInsertRows(self.index, row, row)
 
         # insert the structure
-        orig_model.structures.insert(row - 3, structure)
+        orig_model.structures.insert(row - STRUCT_OFFSET, structure)
         v = 1 / len(orig_model.structures)
         sf = possibly_create_parameter(v, name="scale")
-        orig_model.scales.insert(row - 3, sf)
+        orig_model.scales.insert(row - STRUCT_OFFSET, sf)
 
         self.insertChild(row, n)
         self._model.endInsertRows()
 
         # insert a scale factor
-        self._model.beginInsertRows(self.child(0).index, row - 3, row - 3)
+        self._model.beginInsertRows(
+            self.child(0).index, row - STRUCT_OFFSET, row - STRUCT_OFFSET
+        )
         # add a scale factor
         n = ParNode(sf, self._model, self.child(0))
-        self.child(0).insertChild(row - 3, n)
+        self.child(0).insertChild(row - STRUCT_OFFSET, n)
         self._model.endInsertRows()
 
     def setData(self, column, value, role=QtCore.Qt.EditRole):
@@ -599,13 +624,13 @@ class ReflectModelNode(Node):
             data_object = data_object_node.data_object
             data_object.constantdq_q = self.constantdq_q
 
-            # TODO emit signal for recalculation of chi2. Do this by saying
-            # TODO parnode for constant dq/q has changed.
-
             # need to not let the dq parameter vary if there's resolution
             # information in the dataset
             if not self.constantdq_q:
+                data_object_node.data_object.model.dq_type = "pointwise"
                 data_object_node.data_object.model.dq.vary = False
+            else:
+                data_object_node.data_object.model.dq_type = "constant"
 
             lindex = self._model.index(2, 0, self.index)
             rindex = self._model.index(2, 4, self.index)
@@ -625,7 +650,7 @@ class ReflectModelNode(Node):
 
 class DatasetNode(Node):
     def __init__(self, data, model, parent=QtCore.QModelIndex()):
-        super(DatasetNode, self).__init__(data, model, parent)
+        super().__init__(data, model, parent)
 
     @property
     def dataset(self):
@@ -650,7 +675,7 @@ class DatasetNode(Node):
 
 class DataObjectNode(Node):
     def __init__(self, data_object, model, parent=QtCore.QModelIndex()):
-        super(DataObjectNode, self).__init__(data_object, model, parent=parent)
+        super().__init__(data_object, model, parent=parent)
 
         self.chi2 = float(np.nan)
         self.visible = data_object.graph_properties.visible
@@ -735,6 +760,15 @@ class DataObjectNode(Node):
                 return "points: %d" % len(self._data.dataset)
             elif column == 3:
                 return "chi2: %g" % self.chi2
+        elif role == QtCore.Qt.ToolTipRole:
+            if column == 1:
+                return "Show or hide the dataset from the graphs"
+            elif column == 3:
+                return (
+                    "((y<sub>i,data</sub>-y<sub>i,model</sub>)"
+                    "/y<sub>i,err</sub>)<sup>2</sup> summed over all "
+                    "datapoints"
+                )
 
     def flags(self, column):
         flags = QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
@@ -749,7 +783,7 @@ class DataObjectNode(Node):
 class ContainerNode(Node):
     # set up with a datastore
     def __init__(self, data, model, parent=None):
-        super(ContainerNode, self).__init__(data, model, parent)
+        super().__init__(data, model, parent)
         for data_object in data:
             n = DataObjectNode(data_object, model, parent=self)
             self.appendChild(n)
@@ -825,8 +859,14 @@ class ContainerNode(Node):
 
 
 class TreeModel(QtCore.QAbstractItemModel):
+    """
+    Parameters
+    ----------
+    data: refnx.reflect._app.datastore.DataStore
+    """
+
     def __init__(self, data, parent=None):
-        super(TreeModel, self).__init__(parent)
+        super().__init__(parent)
         self._rootnode = ContainerNode(data, self)
         self._data = data
 
@@ -1059,9 +1099,10 @@ class TreeModel(QtCore.QAbstractItemModel):
 
         # if the snapshot already exists then overwrite it.
         if snapshot_name in self.datastore.names:
-            row = self.datastore.index(snapshot_name)
-            self._rootnode.child[row].set_dataset(dataset)
-            self._rootnode.child[row].set_reflect_model(new_model)
+            row = self.data_object_row(snapshot_name)
+            self._rootnode.child(row).set_dataset(dataset)
+            self._rootnode.child(row).set_reflect_model(new_model)
+            data_object = self.data_object_node(snapshot_name).data_object
         else:
             # otherwise you have to add it.
             data_object = DataObject(dataset)
@@ -1181,13 +1222,13 @@ def find_data_object(index):
 ###############################################################################
 class SlabNode(ComponentNode):
     def __init__(self, data, model, parent=QtCore.QModelIndex()):
-        super(SlabNode, self).__init__(data, model, parent)
+        super().__init__(data, model, parent)
 
 
 ###############################################################################
 class LipidLeafletNode(ComponentNode):
     def __init__(self, data, model, parent=QtCore.QModelIndex()):
-        super(LipidLeafletNode, self).__init__(data, model, parent)
+        super().__init__(data, model, parent)
 
         prop_node = PropertyNode("reverse_monolayer", model, parent=self)
         self.appendChild(prop_node)
@@ -1196,7 +1237,7 @@ class LipidLeafletNode(ComponentNode):
 ###############################################################################
 class SplineNode(ComponentNode):
     def __init__(self, data, model, parent=QtCore.QModelIndex()):
-        super(SplineNode, self).__init__(data, model, parent, flat=False)
+        super().__init__(data, model, parent, flat=False)
         prop_node = PropertyNode("zgrad", model, parent=self)
         self.appendChild(prop_node)
 
@@ -1215,7 +1256,7 @@ class SplineNode(ComponentNode):
 ###############################################################################
 class StackNode(Node):
     def __init__(self, data, model, parent=QtCore.QModelIndex()):
-        super(StackNode, self).__init__(data, model, parent)
+        super().__init__(data, model, parent)
 
         # append the number of repeats
         pn = ParNode(data.repeats, model, parent=self)
@@ -1293,5 +1334,5 @@ class StackNode(Node):
 
         self._model.beginInsertRows(self.index, row, row)
         self.insertChild(row, n)
-        self.stack.insert(row, component)
+        self.stack.insert(row - 1, component)
         self._model.endInsertRows()
