@@ -8,7 +8,9 @@ import csv
 from multiprocessing import get_context
 
 import numpy as np
+import scipy
 import matplotlib
+import periodictable
 
 from PyQt5 import QtCore, QtGui, QtWidgets, uic
 
@@ -43,7 +45,7 @@ from .treeview_gui_model import (
 from ._lipid_leaflet import LipidLeafletDialog
 from ._optimisation_parameters import OptimisationParameterView
 from ._spline import SplineDialog
-from ._mcmc import ProcessMCMCDialog, SampleMCMCDialog, _plots
+from ._mcmc import ProcessMCMCDialog, SampleMCMCDialog, _plots, _process_chain
 
 import refnx
 from refnx.analysis import (
@@ -76,7 +78,7 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
     """
 
     def __init__(self, parent=None):
-        super(MotofitMainWindow, self).__init__(parent)
+        super().__init__(parent)
 
         # load the GUI from the ui file
         self.ui = uic.loadUi(os.path.join(UI_LOCATION, "motofit.ui"), self)
@@ -178,9 +180,10 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
 
         self.restore_settings()
 
+        self.sample_mcmc_dialog = SampleMCMCDialog(self)
         self.spline_dialog = SplineDialog(self)
         self.sld_calculator = SLDcalculatorView(self)
-        self.lipid_leaflet = LipidLeafletDialog(self)
+        self.lipid_leaflet_dialog = LipidLeafletDialog(self)
         self.optimisation_parameters = OptimisationParameterView(self)
         self.data_object_selector = DataObjectSelectorDialog(self)
 
@@ -207,7 +210,9 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
 
         # convert url's to files
         urls_as_files = [url.toLocalFile() for url in urls]
+        self._load_files_different_types(urls_as_files)
 
+    def _load_files_different_types(self, urls_as_files):
         # if you've only got one url then try and load it as an experiment
         if len(urls_as_files) == 1:
             try:
@@ -252,6 +257,7 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
 
         fit_list = self.currently_fitting_model
         state["currently_fitting"] = fit_list.datasets
+        state["requirements.txt"] = self.requirements()
 
         with open(os.path.join(experiment_file_name), "wb") as f:
             pickle.dump(state, f, -1)
@@ -299,6 +305,9 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
             self.restore_settings()
         except KeyError as e:
             print(repr(e))
+            print("\n-------------------------------------------------------")
+            print("\nThese are the packages used when the file was saved:\n")
+            print(state.get("requirements.txt", ""))
             return
 
         try:
@@ -375,6 +384,16 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
         # added Parameter._stderr
         # v0.1.13
 
+        # removes picker key from graphproperties (matplotlib 3.4.0 causes
+        # issues)
+
+        # adds Parameter.units attribute
+        # v0.1.21
+
+        # try and compensate for energy dispersive machinery. This won't be
+        # able to compensate for all possible fails.
+        # v0.1.27
+
         from refnx.analysis.bounds import Interval
 
         for do in self.treeModel.datastore:
@@ -390,7 +409,15 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
                 strcs = [model.structure]
 
             for s in strcs:
+                if not hasattr(s, "wavelength"):
+                    s.wavelength = None
+
                 for component in s:
+                    if isinstance(component, Slab) and isinstance(
+                        component.sld, SLD
+                    ):
+                        component.sld.dispersive = False
+
                     if not hasattr(component, "_interfaces"):
                         component._interfaces = None
 
@@ -408,6 +435,9 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
                             v = parameter.stderr
                         parameter._stderr = v
 
+                    if not hasattr(parameter, "units"):
+                        parameter.units = None
+
                     bnd = parameter.bounds
                     if isinstance(bnd, Interval) and not hasattr(
                         bnd, "_logprob"
@@ -415,6 +445,15 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
                         bnd._logprob = 0
                         bnd._set_bounds(bnd.lb, bnd.ub)
 
+                # pop picker attribute from graphproperties
+                gp = do.graph_properties
+                for line in [
+                    "data_properties",
+                    "fit_properties",
+                    "sld_profile_properties",
+                ]:
+                    gp[line].pop("picker", None)
+
     def apply_settings_to_params(self, params):
         for key in self.settings.__dict__:
             params[key] = self.settings[key]
@@ -566,7 +605,7 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
         # saves an SLD curve as a text file
         datastore = self.treeModel.datastore
 
-        self.data_object_selector.setWindowTitle("Select fits to save")
+        self.data_object_selector.setWindowTitle("Select SLD plots to save")
         ok = self.data_object_selector.exec_()
         if not ok:
             return
@@ -989,7 +1028,7 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
 
         for licence in licences:
             fname = os.path.join(licence_dir, licence)
-            with open(fname, "r", encoding="utf-8", errors="replace") as f:
+            with open(fname, "r", encoding="utf-8") as f:
                 text.append("".join(f.readlines()))
 
         display_text = "\n_______________________________________\n".join(text)
@@ -1002,6 +1041,22 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
             QtCore.QUrl("https://refnx.readthedocs.io/en/latest/")
         )
 
+    @QtCore.pyqtSlot()
+    def on_actionPython_Packages_triggered(self):
+        aboutui = uic.loadUi(os.path.join(UI_LOCATION, "about.ui"))
+        text = self.requirements()
+        aboutui.textBrowser.setText(text)
+        aboutui.exec_()
+
+    def requirements(self):
+        # returns a string of the packages used in the GUI Python environment
+        pkgs = [refnx, np, scipy, matplotlib, periodictable]
+        versions = []
+        for pkg in pkgs:
+            versions.append(f"{pkg.__name__}=={pkg.__version__}")
+
+        return "\n".join(versions)
+
     @QtCore.pyqtSlot()
     def on_actionAutoscale_graph_triggered(self):
         self.reflectivitygraphs.autoscale()
@@ -1016,7 +1071,7 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
 
     @QtCore.pyqtSlot()
     def on_actionLipid_browser_triggered(self):
-        self.lipid_leaflet.show()
+        self.lipid_leaflet_dialog.show()
 
     @QtCore.pyqtSlot()
     def on_add_layer_clicked(self):
@@ -1079,11 +1134,11 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
         if which_type == "Slab":
             c = _default_slab(parent=self)
         elif which_type == "LipidLeaflet":
-            self.lipid_leaflet.hide()
-            ok = self.lipid_leaflet.exec_()
+            self.lipid_leaflet_dialog.hide()
+            ok = self.lipid_leaflet_dialog.exec_()
             if not ok:
                 return
-            c = self.lipid_leaflet.component()
+            c = self.lipid_leaflet_dialog.component()
         elif which_type == "Spline":
             # if isinstance(host, StackNode):
             #     msg("Can't add Splines to a Stack")
@@ -1282,7 +1337,9 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
 
         return objective
 
-    def fit_data_objects(self, data_objects):
+    def fit_data_objects(
+        self, data_objects, alg=None, opt_kws=None, mcmc_kws=None
+    ):
         """
         Simultaneously fits a sequence of datasets
 
@@ -1290,6 +1347,16 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
         ----------
         data_objects: list of DataObjects
 
+        alg: str
+            One of the fitting algorithms. If None then queries the GUI state
+            for the algorithm to use.
+
+        opt_kws: dict
+            Specify options for the optimisation
+
+        mcmc_kws: dict
+            Specify options for the MCMC sampling
+
         Returns
         -------
         fitted_dataobjects : list of DataObjects
@@ -1298,8 +1365,6 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
 
         objective = self.create_objective(data_objects)
 
-        alg = self.settings.fitting_algorithm
-
         # figure out how many varying parameters
         vp = objective.varying_parameters()
         if not vp:
@@ -1314,8 +1379,14 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
             "MCMC": "MCMC",
         }
 
+        if alg not in methods:
+            alg = self.settings.fitting_algorithm
+
         # obtain optimisation parameters (maxiter, etc)
-        kws = self.optimisation_parameters.parameters(alg)
+        if opt_kws is None:
+            kws = self.optimisation_parameters.parameters(alg)
+        else:
+            kws = {}.update(opt_kws)
 
         if methods[alg] != "MCMC":
             fitter = CurveFitter(objective)
@@ -1329,6 +1400,10 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
                 progress.show()
                 kws["callback"] = progress.callback
 
+            if sys.stderr is None:
+                # for pythonw, sys.stderr = None
+                kws["verbose"] = False
+
             try:
                 # workers is added to differential evolution in scipy 1.2
                 if alg == "DE":
@@ -1361,28 +1436,41 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
 
             progress.close()
         else:
-            dialog = SampleMCMCDialog(parent=self)
-            if not dialog.exec_():
-                return []
+            if mcmc_kws is None:
+                ok = self.sample_mcmc_dialog.exec_()
+                if not ok:
+                    return []
+                nwalkers = self.sample_mcmc_dialog.walkers.value()
+                init = self.sample_mcmc_dialog.init.currentText()
+                nsteps = self.sample_mcmc_dialog.steps.value()
+                nthin = self.sample_mcmc_dialog.thin.value()
+                ntemps = self.sample_mcmc_dialog.temps.value()
+                verbose = True
 
-            folder_dialog = QtWidgets.QFileDialog(
-                parent=self, caption="Select location to save MCMC output"
-            )
-            folder_dialog.setFileMode(QtWidgets.QFileDialog.Directory)
-            # folder_dialog.setWindowModality(QtCore.Qt.WindowModal)
-            if folder_dialog.exec_():
-                folder = folder_dialog.selectedFiles()[0]
+                folder_dialog = QtWidgets.QFileDialog(
+                    parent=self, caption="Select location to save MCMC output"
+                )
+                folder_dialog.setFileMode(QtWidgets.QFileDialog.Directory)
+                # folder_dialog.setWindowModality(QtCore.Qt.WindowModal)
+                if folder_dialog.exec_():
+                    folder = folder_dialog.selectedFiles()[0]
+                else:
+                    return []
             else:
-                return []
+                nwalkers = mcmc_kws.get("nwalkers", 200)
+                init = mcmc_kws.get("init", "jitter")
+                nsteps = mcmc_kws.get("nsteps", 100)
+                nthin = mcmc_kws.get("nthin", 1)
+                ntemps = mcmc_kws.get("ntemps", -1)
+                folder = mcmc_kws.get("folder", ".")
+                nplot = mcmc_kws.get("nplot", 200)
+                nburn = mcmc_kws.get("nburn", 0)
+                verbose = mcmc_kws.get("verbose", True)
 
-            nwalkers = dialog.walkers.value()
-            init = dialog.init.currentText()
-            nsteps = dialog.steps.value()
-            nthin = dialog.thin.value()
-            ntemps = dialog.temps.value()
             if ntemps in [-1, 0, 1]:
                 ntemps = -1
-            dialog.close()
+
+            verbose = verbose and sys.stderr is not None
 
             fitter = CurveFitter(objective, ntemps=ntemps, nwalkers=nwalkers)
 
@@ -1417,7 +1505,7 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
                     fitter.sample(
                         nsteps,
                         f=f,
-                        verbose=True,
+                        verbose=verbose,
                         nthin=nthin,
                         callback=callback,
                         pool=workers.map,
@@ -1433,16 +1521,28 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
             progress.close()
 
             # process the samples
+            def close(dialog=None):
+                if hasattr(dialog, "close"):
+                    dialog.close()
+
             try:
-                dialog = ProcessMCMCDialog(
-                    objective, fitter.chain, folder=folder, parent=self
-                )
-                dialog.exec_()
-                dialog.close()
+                if mcmc_kws is None:
+                    dialog = ProcessMCMCDialog(
+                        objective, fitter.chain, folder=folder, parent=self
+                    )
+                    dialog.exec_()
+                    close(dialog)
+                    nplot = dialog.nplot.value()
+                else:
+                    dialog = None
+                    _process_chain(
+                        objective, fitter.chain, nburn, nthin, folder=folder
+                    )
+
                 # create MCMC graphs
-                _plots(objective, nplot=dialog.nplot.value(), folder=folder)
+                _plots(objective, nplot=nplot, folder=folder)
             except Exception as e:
-                dialog.close()
+                close(dialog)
                 msg(repr(e))
                 print(repr(e))
                 return []
@@ -1991,8 +2091,14 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
         if not top_left.isValid():
             return
 
+        # enumerations of the roles are at:
+        # https://doc.qt.io/qt-5/qt.html#ItemDataRole-enum
+
         # find out which data_object / model we're adjusting
         node = top_left.internalPointer()
+        # row = top_left.row()
+        col = top_left.column()
+
         if node is None:
             return
 
@@ -2001,7 +2107,7 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
             and role[0] == QtCore.Qt.CheckStateRole
             and isinstance(node, DataObjectNode)
         ):
-            # set visibility of data_object
+            # you're setting the visibility of a data_object
             graph_properties = node.data_object.graph_properties
             graph_properties.visible = node.visible is True
             self.redraw_data_object_graphs([node.data_object])
@@ -2011,18 +2117,19 @@ class MotofitMainWindow(QtWidgets.QMainWindow):
         wipe_update = []
 
         # redraw if you're altering a PropertyNode (edit or check)
+        # also does constant dq/q <--> pointwise
         if (
-            top_left.column() == 1
+            col == 1
             and len(role)
             and role[0] in [QtCore.Qt.CheckStateRole, QtCore.Qt.EditRole]
-            and isinstance(node, PropertyNode)
+            and isinstance(node, (PropertyNode, ReflectModelNode))
         ):
             wipe_update = [find_data_object(top_left).data_object]
 
         # only redraw if you're altering values
         # otherwise we'd be performing continual updates of the model
         if (
-            top_left.column() == 1
+            col == 1
             and len(role)
             and role[0] == QtCore.Qt.EditRole
             and isinstance(node, ParNode)
@@ -2124,7 +2231,7 @@ class ProgressCallback(QtWidgets.QDialog):
         self.start = time.time()
         self.last_time = time.time()
         self.abort_flag = False
-        super(ProgressCallback, self).__init__(parent)
+        super().__init__(parent)
         self.parent = parent
         self.ui = uic.loadUi(os.path.join(UI_LOCATION, "progress.ui"), self)
         self.elapsed = 0.0
@@ -2167,7 +2274,7 @@ class ProgressCallback(QtWidgets.QDialog):
         return self.abort_flag
 
 
-class ProgramSettings(object):
+class ProgramSettings:
     def __init__(self, **kwds):
         _members = {
             "fitting_algorithm": "DE",
@@ -2223,7 +2330,7 @@ class MyReflectivityGraphs(FigureCanvas):
         self.axes.append(ax)
 
         self.axes[0].autoscale(axis="both", tight=False, enable=True)
-        self.axes[0].set_xlabel("Q")
+        self.axes[0].set_xlabel(r"Q / $\AA^{-1}$")
         self.axes[0].set_ylabel("R")
         # self.axes[0].set_yscale('log')
 
@@ -2322,7 +2429,7 @@ class MyReflectivityGraphs(FigureCanvas):
                 if graph_properties["fit_properties"]:
                     artist.setp(
                         graph_properties.ax_fit,
-                        **graph_properties["fit_properties"]
+                        **graph_properties["fit_properties"],
                     )
 
             # if (dataObject.line2Dresiduals is None and
@@ -2430,7 +2537,9 @@ class MySLDGraphs(FigureCanvas):
                 and data_object.model is not None
             ):
                 try:
-                    sld_profile = data_object.model.structure.sld_profile()
+                    sld_profile = data_object.model.structure.sld_profile(
+                        max_delta_z=1.0
+                    )
 
                     graph_properties.ax_sld_profile.set_data(
                         sld_profile[0], sld_profile[1]
@@ -2478,7 +2587,7 @@ class MySLDGraphs(FigureCanvas):
                 if graph_properties["sld_profile_properties"]:
                     artist.setp(
                         graph_properties.ax_sld_profile,
-                        **graph_properties["sld_profile_properties"]
+                        **graph_properties["sld_profile_properties"],
                     )
 
         self.axes[0].relim()
@@ -2513,7 +2622,7 @@ class OpenMenu(QtWidgets.QMenu):
     """
 
     def __init__(self, parent=None):
-        super(OpenMenu, self).__init__(parent)
+        super().__init__(parent)
         self._parent = parent
         self.copy_from_action = self.addAction("Copy a model to here")
         self.addSeparator()
@@ -2586,7 +2695,7 @@ class CurrentlyFitting(QtCore.QAbstractListModel):
     """
 
     def __init__(self, parent=None):
-        super(CurrentlyFitting, self).__init__(parent=parent)
+        super().__init__(parent=parent)
         self.datasets = []
 
     def rowCount(self, index):
