@@ -318,7 +318,9 @@ class ReflectModel:
         self._dq = possibly_create_parameter(dq, name="dq - resolution")
         self.dq_type = dq_type
 
-        self._q_offset = possibly_create_parameter(q_offset, name="q_offset")
+        self._q_offset = possibly_create_parameter(
+            q_offset, name="q_offset", units="Å**-1"
+        )
 
         self._structure = None
         self.structure = structure
@@ -331,6 +333,7 @@ class ReflectModel:
         ----------
         x : float or np.ndarray
             q values for the calculation.
+            Units = Angstrom**-1
         p : refnx.analysis.Parameters, optional
             parameters required to calculate the model
         x_err : np.ndarray
@@ -419,6 +422,7 @@ class ReflectModel:
         ----------
         x : float or np.ndarray
             q values for the calculation.
+            Units = Angstrom**-1
         p : refnx.analysis.Parameters, optional
             parameters required to calculate the model
         x_err : np.ndarray
@@ -824,7 +828,7 @@ def _smeared_abeles_constant(q, w, resolution, threads=-1):
     gaussgpoint = (gaussnum - 1) / 2
 
     def gauss(x, s):
-        return 1.0 / s / np.sqrt(2 * np.pi) * np.exp(-0.5 * x ** 2 / s / s)
+        return 1.0 / s / np.sqrt(2 * np.pi) * np.exp(-0.5 * x**2 / s / s)
 
     lowq = np.min(q)
     highq = np.max(q)
@@ -973,7 +977,9 @@ class MixedReflectModel:
         self._dq = possibly_create_parameter(dq, name="dq - resolution")
         self.dq_type = dq_type
 
-        self._q_offset = possibly_create_parameter(q_offset, name="q_offset")
+        self._q_offset = possibly_create_parameter(
+            q_offset, name="q_offset", units="Å**-1"
+        )
 
         self._structures = structures
 
