@@ -27,7 +27,7 @@ from refnx.analysis import Parameter, Interval, Parameters
 from refnx.analysis.parameter import _BinaryOp
 
 
-class TestStructure(object):
+class TestStructure:
     def setup_method(self):
         self.air = SLD(0, name="air")
         self.sio2 = SLD(3.47, name="sio2")
@@ -200,6 +200,11 @@ class TestStructure(object):
     def test_sld_profile(self):
         # check that it runs
         z, sld_profile = self.s.sld_profile()
+        assert_equal(np.size(z), 500)
+
+        z, sld_profile = self.s.sld_profile(max_delta_z=0.251)
+        delta_z = np.ediff1d(z)
+        assert delta_z[0] <= 0.251
 
         z, sld_profile = self.s.sld_profile(np.linspace(-100, 100, 100))
         assert_equal(min(z), -100)
@@ -269,6 +274,24 @@ class TestStructure(object):
         p = s.parameters
         assert len(list(flatten(p))) == 5 + 4 + 4
 
+        # check that energy dispersive calculation works
+        au = MaterialSLD(
+            "Au", density=19.33, name="Gold", probe="x-ray", wavelength=1.54
+        )
+        si = SLD(20.1)
+        s = air | au(100, 3) | si(0, 3)
+        slabs = s.slabs(wavelength=0.5)
+        assert_allclose(s.wavelength, 0.5)
+        au_sld = complex(slabs[1, 1], slabs[1, 2])
+
+        au_check = MaterialSLD(
+            "Au", density=19.33, name="Gold", probe="x-ray", wavelength=0.5
+        )
+        assert_allclose(au_sld, complex(au_check))
+
+        # check that the Scatterer.complex method works as intended
+        assert_allclose(au.complex(0.5), complex(au_check))
+
     def test_repr_slab(self):
         p = SLD(5 + 1j)
         t = p(10.5, 3.0)
